@@ -1,4 +1,4 @@
-# Copyright (c) 2020, Riverbank Computing Limited
+# Copyright (c) 2021, Riverbank Computing Limited
 # All rights reserved.
 #
 # This copy of SIP is licensed for use under the terms of the SIP License
@@ -124,10 +124,15 @@ class Project(AbstractProject, Configurable):
         Option('build_dir', help="the build directory", metavar="DIR"),
         Option('build_tag', help="the build tag to be used in the wheel name",
                 metavar="TAG", tools=['wheel']),
+        Option('distinfo', option_type=bool, inverted=True,
+                help="don't create a .dist-info directory", tools=['install']),
         Option('manylinux', option_type=bool, inverted=True,
                 help="disable the use of manylinux in the platform tag used "
                         "in the wheel name",
                 tools=['wheel']),
+        Option('scripts_dir', default=os.path.dirname(sys.executable),
+                help="the scripts installation directory", metavar="DIR",
+                tools=['build', 'install']),
         Option('target_dir', default=get_python_lib(plat_specific=1),
                 help="the target installation directory", metavar="DIR",
                 tools=['build', 'install']),
@@ -146,7 +151,7 @@ class Project(AbstractProject, Configurable):
     def __init__(self, **kwargs):
         """ Initialise the project. """
 
-        super().__init__(**kwargs)
+        super().__init__()
 
         # The current directory should contain the .toml file.
         self.root_dir = os.getcwd()
@@ -159,6 +164,8 @@ class Project(AbstractProject, Configurable):
         self._metadata_overrides = None
         self._temp_build_dir = None
 
+        self.initialise_options(kwargs)
+
     def apply_nonuser_defaults(self, tool):
         """ Set default values for non-user options that haven't been set yet.
         """
@@ -526,6 +533,9 @@ class Project(AbstractProject, Configurable):
             if not b.is_buildable():
                 del self.bindings[b.name]
 
+        if len(self.bindings) == 0:
+            raise UserException("There are no bindings that can be built")
+
     def verify_configuration(self, tool):
         """ Verify that the configuration is complete and consistent. """
 
@@ -576,11 +586,11 @@ class Project(AbstractProject, Configurable):
                     "Python v{}.{} is not supported".format(
                             self.py_major_version, self.py_minor_version))
 
-        # Make sure we have a valid ABI version.
-        self.abi_version = resolve_abi_version(self.abi_version)
+        # Make sure we have a supported ABI version.
+        self.abi_version = resolve_abi_version(self.abi_version, exact=False)
 
         # Checks for standalone projects.
-        if not self.sip_module:
+        if tool in Option.BUILD_TOOLS and not self.sip_module:
             # Check there is only one set of bindings.
             if len(self.bindings) > 1:
                 raise PyProjectOptionException('sip-module',
@@ -681,7 +691,7 @@ class Project(AbstractProject, Configurable):
             for enabled in self.enable:
                 if enabled not in names:
                     raise UserException(
-                            "unknown enabled bindings '{0}'".format(enabled))
+                            "Unknown enabled bindings '{0}'".format(enabled))
 
             # Only include explicitly enabled bindings.
             for b in list(self.bindings.values()):
@@ -693,7 +703,7 @@ class Project(AbstractProject, Configurable):
             for disabled in self.disable:
                 if disabled not in names:
                     raise UserException(
-                            "unknown disabled bindings '{0}'".format(disabled))
+                            "Unknown disabled bindings '{0}'".format(disabled))
 
             # Remove any explicitly disabled bindings.
             for b in list(self.bindings.values()):
